//	BarsState.h - State for the 'Blade in my Chest' branch.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef BARSSTATE_H_
#define BARSSTATE_H_

#include "ThreeFloats.h"
#include "GameState.h"

#include <vector>

class PhaseDriftSounder;

///	State for the 'Blade in my Chest' branch.
class BarsState : public GameState,
				  public StreamingSound::Callback
{
  public:
	///	Constructor.
	BarsState(Application& app, SounderPtr& sound);
	///	Destructor.
	~BarsState();

	///	Moves the bars.
	void update();
	///	Draws the bars.
	void draw(DrawerPtr& d, float timestep);

	///	Called when a key down event is received.
	void keyDown(const std::wstring& action,
				 const std::wstring& keyPressed,
				 int keyModifier);

	///	Called when a mouse move event is received.
	void mouseMove(const TwoFloats& pos);

	///	Called to trigger the BarsState when we've reached a particular point in Music_BarsDrone.
	void reachedTime(const std::wstring& name, float time);

	///	Returns the name of this state.
	const std::wstring getName() const {return L"BarsState";};
	///	Used to construct instances of this state.
	static GameStateCreator<BarsState> creator;
  private:
	///	The number of colour spaces we have.
	enum
	{
		NumColourSpaces = 4
	};

	///	Struct representing a single bar.
	struct Bar
	{
		///	The bar's x-axis position.
		float pos;
		///	The bar's direction (& speed).
		float dir;
		///	The bar's colour.
		ThreeFloats col;
		///	The bar's image.
		std::wstring image;
		///	The bar's width.
		float width;

		///	The bar's colours in the different colour spaces.
		/*!
			0 = Black & white.
			1 = Black, red & white.
			2 = Pastel blue, yellow, green, pink.
			3 = Random.
		 */
		ThreeFloats colourSpaceCols[NumColourSpaces];
	};

	///	Helper method. Updates a bar's colours.
	void updateBarColours(Bar& bar);
	///	Helper method. Interpolates a bar's colour.
	void calcInterpColour(Bar& bar);

	///	Temp.
	bool tempBar;
	///	Temp.
	float rotVal;

	///	Used to control the speed of rotVal.
	float rotSpeed;
	///	Used to control the speed of rotVal.
	float rotSpeedDest;

	///	All the active bars.
	std::vector<Bar> bars;

	///	The index to the current colour space.
	float colourSpaceIndex;
	///	Used to interpolate colour spaces over time.
	float destColourSpaceIndex;

	///	Index to the background drone.
	int droneIndex;

	///	Our sounder.
	shared_ptr<PhaseDriftSounder> sounder;
};

#endif
