//	OctopusState.cpp - The state with 'octopus' arms.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "drawing/Drawer.h"
#include "OctopusState.h"
#include "MiscHelper.h"
#include "Random.h"

using std::vector;

//------------------------------------------------------------------------------
GameStateCreator<OctopusState> OctopusState::creator;

//------------------------------------------------------------------------------
OctopusState::OctopusState(Application& app, SounderPtr& sound):
GameState(app, sound),
newDotTimer(1),
mousePos(640.0f, 360.0f),
centrePos(640.0f, 360.0f)
{
	int i;

	for(i=0;i<NumArms;++i)
	{
		arms[i].angle = (float)i/(float)NumArms;
		//arms[i].speed = 0.01f + (Random::getGlobal().getRandomFloat() * 0.29f);
		arms[i].speed = 0.0f;
		//if(Random::getGlobal().getRandomFloat() < 0.5f)
			arms[i].speed = -arms[i].speed;
		arms[i].sinSpeed[0] = 1.0f + (Random::getGlobal().getRandomFloat() * 3.0f);
		arms[i].sinSpeed[1] = 1.0f + (Random::getGlobal().getRandomFloat() * 3.0f);
		arms[i].sinAmp[0] = 0.5f + (Random::getGlobal().getRandomFloat() * 1.5f);
		arms[i].sinAmp[1] = 0.5f + (Random::getGlobal().getRandomFloat() * 1.5f);
		arms[i].sinOffset[0] = 0.0f;
		arms[i].sinOffset[1] = 0.0f;
	}

	setMousePos(mousePos);
}

//------------------------------------------------------------------------------
OctopusState::~OctopusState()
{

}

//------------------------------------------------------------------------------
void OctopusState::update()
{
	int i;
	float tempf;
	float tempAmp;
	TwoFloats lastPos;
	TwoFloats tempPos;
	TwoFloats tempDir;
	float mouseSpeed = 0.0f;
	const TwoFloats& drawSize = getDrawingAreaSize();

	//Update centre position.
	if(centrePos != mousePos)
	{
		centrePos += (mousePos-centrePos) * 0.25f;
		mouseSpeed = mousePos.getDistance(centrePos);
		if(mouseSpeed < 0.5f)
			centrePos = mousePos;
	}

	mouseSpeed *= 1.0f/(1280.0f * 2.0f);

	//Update the arms.
	for(i=0;i<NumArms;++i)
	{
		arms[i].angle += (arms[i].speed * 0.01f);
		if(arms[i].angle >= 1.0f)
			arms[i].angle -= 1.0f;

		/*if(arms[i].speed >= 0.0f)
		{
			arms[i].speed += mouseSpeed;
			while(arms[i].speed > 1.0f)
				arms[i].speed -= 1.0f;
		}
		else*/
		{
			arms[i].speed -= mouseSpeed;
			while(arms[i].speed < -1.0f)
				arms[i].speed += 1.0f;
		}

		arms[i].sinOffset[0] += 0.005f;
		/*if(arms[i].sinOffset[0] > 1.0f)
			arms[i].sinOffset[0] -= 1.0f;*/

		arms[i].sinOffset[1] += 0.01f;
		/*if(arms[i].sinOffset[1] > 1.0f)
			arms[i].sinOffset[1] -= 1.0f;*/
	}

	//Update existing dots.
	for(i=(dots.size()-1);i>=0;--i)
	{
		if(dots[i].armPos < 1.0f)
		{
			//Work out where a dot should be on its Arm.
			//--Work out the arm's direction vector.
			tempf = arms[dots[i].armIndex].angle * 2.0f * NIALL_PI;
			tempDir.x = sinf(tempf);
			tempDir.y = cosf(tempf);

			//--Work out the dot's amplitude on the direction vector.
			tempAmp = sinf(arms[dots[i].armIndex].sinOffset[0] + dots[i].armPos * 2.0f * NIALL_PI * arms[dots[i].armIndex].sinSpeed[0]) * arms[dots[i].armIndex].sinAmp[0];
			tempAmp += sinf(arms[dots[i].armIndex].sinOffset[1] + dots[i].armPos * 2.0f * NIALL_PI * arms[dots[i].armIndex].sinSpeed[1]) * arms[dots[i].armIndex].sinAmp[1];

			tempPos = centrePos + (tempDir * dots[i].armPos * 512.0f);
			tempDir = tempDir.perpendicular();
			tempf = dots[i].armPos * 4.0f;
			if(tempf > 1.0f)
				tempf = 1.0f;
			tempPos += tempDir * tempAmp * 8.0f * tempf;

			//Move the dot towards the intended position.
			lastPos = dots[i].position;
			dots[i].position += (tempPos-dots[i].position) * (0.125f * (1.0f-dots[i].armPos));
			dots[i].direction = (tempPos-dots[i].position).normal().perpendicular();
			dots[i].speed = lastPos.getDistance(dots[i].position);

			//Update the dot's position on the arm.
			dots[i].armPos += 0.001f;

			//Update the dot's alpha.
			if(dots[i].alpha < 1.0f)
				dots[i].alpha += 0.02f;
		}
		else
		{
			if(dots[i].alpha > 0.0f)
				dots[i].alpha -= 0.01f;
			if(dots[i].speed < 4.0f)
				dots[i].speed += 0.05f;

			dots[i].position += dots[i].direction * dots[i].speed;
			if((dots[i].position.x < -64.0f) ||
			   (dots[i].position.x >= (drawSize.x+64.0f)) ||
			   (dots[i].position.y < -64.0f) ||
			   (dots[i].position.y >= (drawSize.y+64.0f)))
			{
				dots.erase(dots.begin()+i);
			}
		}
	}

	//Add new dots.
	--newDotTimer;
	if(!newDotTimer)
	{
		for(i=0;i<NumArms;++i)
		{
			dots.push_back(Dot());
			dots.back().armIndex = i;
			dots.back().position = centrePos;
		}

		newDotTimer = 60;
	}
}

//------------------------------------------------------------------------------
void OctopusState::draw(DrawerPtr& d, float timestep)
{
	float tempf;
	unsigned int i;
	ThreeFloats tempCol;
	//const ThreeFloats col0(0.05f, 0.4f, 0.05f);
	const ThreeFloats col0(0.66f, 0.66f, 1.0f);
	const ThreeFloats col1(0.66f, 0.3f, 0.1f);

	//d->fillRect(TwoFloats(0.0f), d->getDrawingAreaSize(), ThreeFloats(0.2f, 0.66f, 0.2f));

	for(i=0;i<dots.size();++i)
	{
		tempf = dots[i].armPos;
		if(tempf > 1.0f)
			tempf = 1.0f;

		tempCol = (col0 * (1.0f-tempf)) + (col1 * tempf);

		d->drawImage(L"Point",
					 dots[i].position-(32.0f*0.66f),
					 TwoFloats(0.66f),
					 tempCol,
					 /*0.75f */ dots[i].alpha);
	}
}

//------------------------------------------------------------------------------
void OctopusState::keyDown(const std::wstring& action,
						   const std::wstring& keyPressed,
						   int keyModifier)
{
	if(action == L"Escape")
		quit();
}

//------------------------------------------------------------------------------
void OctopusState::mouseMove(const TwoFloats& pos)
{
	mousePos = pos;
}
