//	PhaseDriftSounder.h - The Sounder for my Phase Drift game.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef PHASEDRIFTSOUNDER_H_
#define PHASEDRIFTSOUNDER_H_

#include "sound/StreamingSound.h"
#include "InterestingNoise.h"
#include "sound/Sounder.h"
#include "StuckBuffer.h"
#include "MiscHelper.h"
#include "TwoFloats.h"
#include "Thread.h"

#include <string>
#include <map>

///	A Sounder which plays StreamingSounds.
class PhaseDriftSounder : public Sounder,
						  public Thread
{
  public:
	///	Constructor.
	PhaseDriftSounder();
	///	Destructor.
	~PhaseDriftSounder();
	
	///	Called for every audio block.
	void getAudio(float *input, float *output, int numSamples);

	///	Called for each sound file resource by LoadingScreen.
	void loadSoundFileResource(SoundFileLoaderPtr soundFileLoader);

	///	Used to stream audio off the disk.
	bool run();

	///	Starts a StreamingSound playing at the desired position in the scene.
	/*!
		\param sound The name of the sound to play.
		\param pos The sound's position in the scene.
		\return The index of the sound which has been activated, so you can
		call deactivateSound() on it later.

		The sound will play until it reaches its end. Its level will be
		determined by the distance between it and the player.
	 */
	int activateSound(const std::wstring& sound, const TwoFloats& pos);
	///	Stops the named sound playing w/a fade out.
	void deactivateSound(int index);
	///	Stops all sounds currently playing (w/a fade out).
	void killAllSounds();
	///	Triggers a StreamingSound to play as a one-shot.
	/*!
		\param The name of the sound to play.
		\param pan How the sound is panned (0->1; left->right).

		This will tell a StreamingSound to play, and stop when it reaches the
		end of its file.
	 */
	void triggerSound(const std::wstring& sound, float pan = 0.5f);

	///	Registers a callback with the indexed sound.
	void registerCallback(int soundIndex,
						  float time,
						  StreamingSound::Callback *callback);
	///	Unregisters a callback with the indexed sound.
	void unregisterCallback(int soundIndex,
							StreamingSound::Callback *callback);

	///	Sets the player's position within the scene.
	void setPlayerPos(const TwoFloats& pos);
	///	Sets the amount the noise encroaches on the other sounds.
	/*!
		Note: This should never be 0.

		1: Other sounds are heard clearly.
		(close to) 0: Noise is overwhelming (but other sounds are still somewhat
		audible).
	 */
	void setNoiseLevel(float val);
	///	Sets the noise's ramp value.
	void setNoiseRamp(float val);
	///	Stops the noise.
	void killNoise();

	///	Stops the stuck buffer effect.
	void killStuckBuffer();
  private:
	///	Calculates the level of a sound relative to the player's position.
	inline float getLevel(const TwoFloats& soundPos) const
	{
		float retval = 0.0f;
		const float curDist = maxDist * noiseLevel;
		const float dist = playerPos.getDistance(soundPos);

		if(dist < curDist)
			retval = fakeLog(1.0f-(dist/curDist));

		return retval;
	};

	///	Struct representing a pre-loaded buffer.
	struct PreLoadedBuffer
	{
		///	Pointer to the buffer data.
		float *buffer;
		///	The size of the buffer data.
		int size;
	};

	///	The number of available StreamingSounds.
	enum
	{
		NumSounds = 128
	};

	///	Map of sound file names and their associated SoundFileLoaders.
	/*!
		Used for longer files.
	 */
	std::map<std::wstring, SoundFileLoaderPtr> soundFiles;
	///	Map of sound file names and their pre-loaded buffers.
	/*!
		Used for short files, which can reasonably be read entirely into
		memory.
	 */
	std::map<std::wstring, PreLoadedBuffer> preLoadedBuffers;

	///	The available StreamingSounds.
	StreamingSound sounds[NumSounds];
	///	The positions of the active StreamingSounds.
	TwoFloats positions[NumSounds];

	///	The player's position in the scene (0->1).
	TwoFloats playerPos;
	///	How much the noise is encroaching on the other sounds.
	float noiseLevel;
	///	The maximum level beyond which sounds are inaudible.
	float maxDist;

	///	The interference noise.
	InterestingNoise noise;
	///	The stuck buffer effect.
	StuckBuffer stuckBuffer;

	///	Whether the background noise is currently on.
	bool noiseOn;
	///	Whether the stuck buffer effect is currently on.
	bool stuckBufferOn;

	///	Used to kill the thread.
	bool timeToStop;
};

#endif
