//	PhaseDriftState.cpp - The main gameplay state.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "PhaseDriftSounder.h"
#include "PhaseDriftState.h"
#include "drawing/Drawer.h"
#include "MiscHelper.h"
#include "UTF8File.h"

#include <cassert>
#include <cmath>

using namespace std;

//------------------------------------------------------------------------------
GameStateCreator<PhaseDriftState> PhaseDriftState::creator;

//------------------------------------------------------------------------------
PhaseDriftState::PhaseDriftState(Application& app, SounderPtr& sound):
GameState(app, sound),
rot(0.0f),
rotSpeed(0.002f),
mousePos(0.5f, 0.5f),
fadeIn(1.0f),
currentSource(-1),
staticCount(0),
foundSource(false),
backFade(0.0f),
bounds(0.0f),
bladeFade(1.0f),
storiesCount(300),
bladeCount(300),
phaseDrift2Count(3000),
sounder(dynamic_pointer_cast<PhaseDriftSounder>(sound))
{
	sourcesPos.push_back(TwoFloats(0.5f));
	sourcesIndices.push_back(sounder->activateSound(L"Part1_Introduction",
													sourcesPos[0]));
	sourcesPos.push_back(TwoFloats(0.1f, 0.2f));
	sourcesIndices.push_back(sounder->activateSound(L"Part2_TheBladeInMyChest",
													sourcesPos[1]));
	sourcesPos.push_back(TwoFloats(0.8f, 0.95f));
	sourcesIndices.push_back(sounder->activateSound(L"Part3_OurStoriesAreWrong",
													sourcesPos[2]));
	sourcesPos.push_back(TwoFloats(0.15f, 0.9f));
	sourcesIndices.push_back(sounder->activateSound(L"Part4_KandinskyColourForm",
													sourcesPos[3]));
	sourcesPos.push_back(TwoFloats(0.95f, 0.55f));
	sourcesIndices.push_back(sounder->activateSound(L"Part5_Hints",
													sourcesPos[4]));
	sourcesPos.push_back(TwoFloats(0.33f, 0.45f));
	sourcesIndices.push_back(sounder->activateSound(L"NumberStation_letter_ys",
													sourcesPos[5]));
	sourcesPos.push_back(TwoFloats(0.56f, 0.7f));
	sourcesIndices.push_back(sounder->activateSound(L"NumberStation_oblique",
													sourcesPos[6]));
	sourcesPos.push_back(TwoFloats(0.75f, 0.15f));
	sourcesIndices.push_back(sounder->activateSound(L"NumberStation_ready_ready",
													sourcesPos[7]));
	sourcesPos.push_back(TwoFloats(0.9f, 0.33f));
	sourcesIndices.push_back(sounder->activateSound(L"NumberStation_rough_tones",
													sourcesPos[8]));
}

//------------------------------------------------------------------------------
PhaseDriftState::~PhaseDriftState()
{

}

//------------------------------------------------------------------------------
void PhaseDriftState::draw(DrawerPtr& d, float timestep)
{
	float tempf;
	float index;
	float radius;
	TwoFloats pos;
	TwoFloats scale(0.33f);
	const TwoFloats origin;
	ThreeFloats col1(0.1f, 0.1f, 1.0f);
	ThreeFloats col2(1.0f, 0.1f, 0.1f);
	const wstring tempstr(L"Point");
	const TwoFloats centre = d->getDrawingAreaSize()*0.5f;

	static float dist = 0.0f;
	//float dist2 = sinf(dist * 6.283185f) * 0.001f;
	float scaleExtra;
	float rad = 16.0f + (mousePos.x * 40.0f);

	if(drawSize == 0.0f)
		drawSize = d->getDrawingAreaSize();

	//Background.
	d->fillRect(origin, drawSize, backColour);

	//Blue points.
	for(radius=0.0f;radius<512.0f;radius+=rad)
	{
		scaleExtra = (radius/512.0f) * 4.0f * mousePos.y;
		scaleExtra += (1.0f-(radius/512.0f)) * 2.0f * (1.0f-mousePos.y);
		scaleExtra *= bladeFade;

		for(index=bounds;index<1.0f;index+=(1.0f/32.0f))
		{
			pos.x = centre.x + (sinf((index+(rot*((radius)/512.0f))) * 6.283185f) * radius * bladeFade);
			pos.y = centre.y + (cosf((index+(rot*((radius)/512.0f))) * 6.283185f) * radius * bladeFade);

			d->drawImage(tempstr,
						 pos - (32.0f * scale.x * scaleExtra),
						 scale * scaleExtra,
						 col1,
						 0.75f);

			if((index == 0.0f) && (radius == 0.0f))
				break;
		}
	}

	//Red points.
	for(radius=0.0f;radius<512.0f;radius+=rad)
	{
		scaleExtra = (radius/512.0f) * 4.0f * mousePos.y;
		scaleExtra += (1.0f-(radius/512.0f)) * 2.0f * (1.0f-mousePos.y);
		scaleExtra *= bladeFade;

		for(index=0.0f;index<(1.0f-bounds);index+=(1.0f/32.0f))
		{
			pos.x = centre.x + (sinf((index+dist*(rot*((radius)/512.0f))) * 6.283185f) * radius * bladeFade);
			pos.y = centre.y + (cosf((index+dist*(rot*((radius)/512.0f))) * 6.283185f) * radius * bladeFade);

			d->drawImage(tempstr,
						 pos - (32.0f * scale.x * scaleExtra),
						 scale * scaleExtra,
						 col2,
						 0.75f);

			if((index == 0.0f) && (radius == 0.0f))
				break;
		}
	}

	//If we're taking the 'Blade...' branch, fade in the big circle.
	if(bladeFade < 1.0f)
	{
		tempf = (1.0f-bladeFade) * 0.05f;
		d->drawImage(L"BigCircle",
					 centre-(tempf * 1024.0f),
					 tempf,
					 ThreeFloats(1.0f, (25.0f/255.0f), (26.0f/255.0f)));
	}

	if(fadeIn > 0.0f)
	{
		d->fillRect(0.0f, d->getDrawingAreaSize(), 0.0f, fadeIn);

		fadeIn -= 0.02f;
	}

	//rotSpeed = 0.002f * ((mousePos.x-0.5f)*2.0f);

	rot += rotSpeed;
	/*if(rot > 2.0f)
		rot -= 2.0f;
	else if(rot < 0.0f)
		rot += 2.0f;*/

	sounder->setNoiseLevel((cosf(rot * 3.14159f)*0.4f)+0.4f+0.2f);

	dist += 0.0005f;
	if(dist > 1.0f)
		dist -= 1.0f;

	//Check if we should be moving the mouse towards a sound source.
	if(!foundSource)
	{
		++staticCount;
		if(staticCount > 120)
		{
			unsigned int i;

			//If we don't have a source yet, find one.
			if(currentSource < 0)
			{
				for(i=0;i<sourcesPos.size();++i)
				{
					if(mousePos.getDistance(sourcesPos[i]) < 0.25f)
					{
						currentSource = i;
						break;
					}
				}
			}

			if(currentSource > -1)
			{
				mousePos += (sourcesPos[currentSource] - mousePos).normal() * 0.01f;
				if(mousePos.getDistance(sourcesPos[currentSource]) < 0.01f)
				{
					foundSource = true;
					mousePos = sourcesPos[currentSource];
				}
				sounder->setPlayerPos(mousePos);
			}
			else
				foundSource = true;
		}
	}

	//Update backColour if the player follows that branch.
	if((backFade > 0.0f) && (backFade < 0.75f))
	{
		backFade += 0.001f;
		backColour.z = backFade;

		if(backFade >= 0.74f)
			bounds += 0.001f;
	}

	if(bounds > 0.0f)
	{
		bounds += 0.001f;
		if(bounds >= 1.0f)
		{
			nextState(L"StoriesState");
			return;
		}
	}

	//Used to determine when to start fading to blue.
	if(currentSource == 2)
	{
		--storiesCount;
		if(storiesCount <= 0)
		{
			backFade += 0.001f;

			sounder->deactivateSound(sourcesIndices[0]);
			sounder->deactivateSound(sourcesIndices[1]);
			sounder->deactivateSound(sourcesIndices[3]);
			sounder->deactivateSound(sourcesIndices[4]);
			sounder->deactivateSound(sourcesIndices[5]);
			sounder->deactivateSound(sourcesIndices[6]);
		}
	}
	//Used to determine when to move to the 'Blade in my Chest' branch.
	else if(currentSource == 1)
	{
		--bladeCount;
		if(bladeCount <= 0)
		{
			bladeFade -= 0.001f;

			if(bladeFade < 0.0f)
			{
				nextState(L"BladeState");
				return;
			}
		}
	}

	//Change to PhaseDrift2 state after a while.
	if(backFade == 0.0f)
	{
		--phaseDrift2Count;
		if(phaseDrift2Count <= 0)
		{
			sounder->killAllSounds();
			sounder->killNoise();
			sounder->killStuckBuffer();
			nextState(L"PhaseDriftState2");
		}
		else if(phaseDrift2Count < 300)
		{
			sounder->setNoiseRamp(4.0f * fakeExp(1.0f - ((float)phaseDrift2Count/300.0f)));
			//backColour = fakeExp(1.0f - ((float)phaseDrift2Count/300.0f));
		}
	}
}

//------------------------------------------------------------------------------
void PhaseDriftState::keyDown(const wstring& action,
							const wstring& keyPressed,
							int keyModifier)
{
	if(action == L"Escape")
		quit();
#ifdef DEBUG
	else if(action == L"P1Right")
	{
		sounder->killAllSounds();
		sounder->killNoise();
		sounder->killStuckBuffer();
		nextState(L"PhaseDriftState2");
	}
	else if(action == L"P1Left")
	{
		mousePos = sourcesPos[2];
		sounder->setPlayerPos(mousePos);
		nextState(L"StoriesState");
	}
	else if(action == L"P1Up")
	{
		/*mousePos = sourcesPos[1];
		sounder->setPlayerPos(mousePos);
		nextState(L"BladeState");*/
		sounder->killAllSounds();
		sounder->killNoise();
		sounder->killStuckBuffer();
		nextState(L"BarsState");
	}
	else if(action == L"P1Down")
	{
		sounder->killAllSounds();
		sounder->killNoise();
		sounder->killStuckBuffer();
		nextState(L"OctopusState");
	}
#endif
}

//------------------------------------------------------------------------------
void PhaseDriftState::mouseMove(const TwoFloats& pos)
{
	if(backFade == 0.0f)
	{
		mousePos = pos/drawSize;

		sounder->setPlayerPos(mousePos);

		currentSource = -1;
		staticCount = 0;
		foundSource = false;
		storiesCount = 300;
	}
}
