//	PhaseDriftState2.cpp - Second state stealing from Side Phase Drift.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "PhaseDriftSounder.h"
#include "PhaseDriftState2.h"
#include "drawing/Drawer.h"
#include "MiscHelper.h"
#include "Random.h"

using namespace std;

//------------------------------------------------------------------------------
GameStateCreator<PhaseDriftState2> PhaseDriftState2::creator;

//------------------------------------------------------------------------------
PhaseDriftState2::PhaseDriftState2(Application& app, SounderPtr& sound):
GameState(app, sound),
mousePos(0.5f, 0.5f),
fadeIn(1.0f),
currentImage(L"Circle"),
primitiveFirst(0),
xCol(4.0f),
whiteFlash(0.0f),
nextStateReady(0),
sounder(dynamic_pointer_cast<PhaseDriftSounder>(sound))
{
	bool foundHigh = false;

	sounder->setPlayerPos(TwoFloats(0.5f));
	musicIndex = sounder->activateSound(L"Music_PhaseDrift2", TwoFloats(0.5f));
	sounder->registerCallback(musicIndex, 45.66f, this);

	backgroundColour.x = Random::getGlobal().getRandomFloat();
	if(backgroundColour.x > 0.5f)
		foundHigh = true;
	backgroundColour.y = Random::getGlobal().getRandomFloat();
	if(backgroundColour.y > 0.5f)
		foundHigh = true;
	if(!foundHigh)
		backgroundColour.z = 0.5f + (Random::getGlobal().getRandomFloat() * 0.5f);
	else
		backgroundColour.z = Random::getGlobal().getRandomFloat();
	desiredColour = backgroundColour;

	potentialColours[0] = ThreeFloats(0.8f, 0.1f, 0.0f);	//Red
	potentialColours[1] = ThreeFloats(0.75f, 0.5f, 0.0f);
	potentialColours[2] = ThreeFloats(0.75f, 0.75f, 0.1f);	//Yellow
	potentialColours[3] = ThreeFloats(0.1f, 0.5f, 0.1f);
	potentialColours[4] = ThreeFloats(0.0f, 0.75f, 0.5f);
	potentialColours[5] = ThreeFloats(0.1f, 0.33f, 0.66f);
	potentialColours[6] = ThreeFloats(0.1f, 0.1f, 0.7f);	//Blue
	potentialColours[7] = ThreeFloats(0.4f, 0.0f, 0.5f);

	primitives[0].image = currentImage;
	primitives[0].scale = 0.0f;
	primitives[0].colour = potentialColours[0];
	primitives[1].image = currentImage;
	primitives[1].scale = -(2.0f*NIALL_PI)/3.0f;
	primitives[1].colour = potentialColours[1];
	primitives[2].image = currentImage;
	primitives[2].scale = 2.0f * -(2.0f*NIALL_PI)/4.0f;
	primitives[2].colour = potentialColours[2];
	primitives[3].image = currentImage;
	primitives[3].scale = 3.0f * -(2.0f*NIALL_PI)/4.0f;
	primitives[3].colour = potentialColours[3];
	primitives[4].image = currentImage;
	primitives[4].scale = 4.0f * -(2.0f*NIALL_PI)/4.0f;
	primitives[4].colour = potentialColours[4];
	primitives[5].image = currentImage;
	primitives[5].scale = 5.0f * -(2.0f*NIALL_PI)/4.0f;
	primitives[5].colour = potentialColours[5];
	primitives[6].image = currentImage;
	primitives[6].scale = 6.0f * -(2.0f*NIALL_PI)/4.0f;
	primitives[6].colour = potentialColours[6];
	primitives[7].image = currentImage;
	primitives[7].scale = 7.0f * -(2.0f*NIALL_PI)/4.0f;
	primitives[7].colour = potentialColours[7];

	images.push_back(L"Circle");
	images.push_back(L"Square");
	images.push_back(L"Triangle");
}

//------------------------------------------------------------------------------
PhaseDriftState2::~PhaseDriftState2()
{
	sounder->unregisterCallback(musicIndex, this);
}

//------------------------------------------------------------------------------
void PhaseDriftState2::draw(DrawerPtr& d, float timestep)
{
	int i, j;
	float alpha;
	float scale;
	bool beforePi;
	const TwoFloats origin;
	const ThreeFloats black;
	const ThreeFloats white(1.0f);
	const TwoFloats centre(d->getDrawingAreaSize().x*0.5f, d->getDrawingAreaSize().y*0.5f);

	if(drawSize == 0.0f)
		drawSize = d->getDrawingAreaSize();

	//Update background colour if necessary.
	if(backgroundColour != desiredColour)
	{
		backgroundColour += (desiredColour - backgroundColour) * 0.05f;
		if(backgroundColour.getDistance(desiredColour) < 0.1f)
			backgroundColour = desiredColour;
	}

	//Background.
	d->fillRect(origin, drawSize, backgroundColour);

	for(i=0,j=primitiveFirst;i<8;++i)
	{
		if(primitives[j].scale > 0.0f)
		{
			scale = 1.0f - ((cosf(primitives[j].scale)*0.5f)+0.5f);

			d->drawImage(primitives[j].image,
						 centre-(256.0f * scale),
						 TwoFloats(scale),
						 primitives[j].colour,
						 0.8f);

			if(primitives[j].scale < NIALL_PI)
				beforePi = true;
			else
				beforePi = false;

			primitives[j].scale += 0.02f;
			if(beforePi && (primitives[j].scale >= NIALL_PI))
			{
				//Check if this primitive & background colour matches one of the
				//3 goal conditions.
				if(primitiveMatches(primitives[j].image, primitives[j].colour))
				{
					unsigned int k;

					whiteFlash = 0.1f;

					//If images only has 1 member, then it's time to move on to
					//the next state.
					if(images.size() == 1)
					{
						/*quit(); //temp.
						return;*/
						nextStateReady = 1;
					}
					else
					{
						//Remove the image that was just matched.
						for(k=0;k<images.size();++k)
						{
							if(images[k] == primitives[j].image)
								break;
						}
						images.erase(images.begin()+k);

						//Now switch all the primitives to use one of the remaining
						//images.
						for(k=0;k<8;++k)
						{
							if(k != j)
							{
								if(primitives[k].image == primitives[j].image)
									primitives[k].image = images[0];
							}
						}
						primitives[j].image = images[0];
						currentImage = images[0];
					}
				}
			}
			else if(primitives[j].scale >= (2.0f*NIALL_PI))
			{
				primitives[j].scale -= 4.0f * NIALL_PI;
				++primitiveFirst;
				primitiveFirst %= 8;
			}
		}
		else
		{
			primitives[j].scale += 0.02f;
			if(primitives[j].scale >= 0.0f)
				primitives[j].image = currentImage;
		}
		++j;
		j %= 8;
	}

	if(fadeIn > 0.0f)
	{
		//Fade in.
		d->fillRect(origin, drawSize, black, fadeIn);

		fadeIn -= 0.01f;
	}

	//Used to indicate the player matched one of the goal conditions.
	if(whiteFlash > 0.0f)
	{
		alpha = 1.0f - ((cosf(whiteFlash)*0.5f)+0.5f);

		d->fillRect(origin, drawSize, white, alpha);

		whiteFlash += 0.1f;
		if(whiteFlash >= (2.0f*NIALL_PI))
			whiteFlash = 0.0f;
		else if(nextStateReady && (whiteFlash >= NIALL_PI))
		{
			switch(nextStateReady)
			{
				case 1:
					quit();
					break;
				case 2:
					nextState(L"BarsState");
					break;
			}
		}
	}
}

//------------------------------------------------------------------------------
void PhaseDriftState2::keyDown(const wstring& action,
							   const wstring& keyPressed,
							   int keyModifier)
{
	if(action == L"Escape")
		quit();
}

//------------------------------------------------------------------------------
void PhaseDriftState2::mouseMove(const TwoFloats& pos)
{
	int index;
	float tempf;

	mousePos = pos/getDrawingAreaSize();

	index = (int)(mousePos.x * 6.9999f);
	xCol = mousePos.x * 6.9999f;
	tempf = xCol - (float)index;

	desiredColour = potentialColours[index] * (1.0f-tempf);
	desiredColour += potentialColours[index+1] * tempf;

	index = (int)(mousePos.y * ((float)images.size()*0.9999f));
	currentImage = images[index];
}

//------------------------------------------------------------------------------
void PhaseDriftState2::reachedTime(const std::wstring& name, float time)
{
	if((name == L"Music_PhaseDrift2") && (time == 45.66f))
	{
		whiteFlash = 0.1f;
		nextStateReady = 2;
	}
}

//------------------------------------------------------------------------------
bool PhaseDriftState2::primitiveMatches(const wstring& image,
										const ThreeFloats& col)
{
	bool retval = false;
	const ThreeFloats yellow(0.75f, 0.75f, 0.1f);
	const ThreeFloats red(0.8f, 0.1f, 0.0f);
	const ThreeFloats blue(0.1f, 0.1f, 0.7f);

	if(image == L"Triangle")
	{
		if(col == yellow)
		{
			if(fabsf(xCol-2.0f) < 0.5f)
				retval = true;
		}
	}
	else if(image == L"Square")
	{
		if(col == red)
		{
			if(xCol < 0.5f)
				retval = true;
		}
	}
	else if(image == L"Circle")
	{
		if(col == blue)
		{
			if(fabsf(xCol-6.0f) < 0.5f)
				retval = true;
		}
	}

	return retval;
}
