//	PhaseDriftState2.h - Second state stealing from Side Phase Drift.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef PHASEDRIFTSTATE2_H_
#define PHASEDRIFTSTATE2_H_

#include "sound/StreamingSound.h"
#include "ThreeFloats.h"
#include "c0xHeaders.h"
#include "GameState.h"

class PhaseDriftSounder;

///	Second state stealing from Side Phase Drift.
class PhaseDriftState2 : public GameState,
						 public StreamingSound::Callback
{
  public:
	///	Constructor.
	PhaseDriftState2(Application& app, SounderPtr& sound);
	///	Destructor.
	~PhaseDriftState2();

	///	Draws the state.
	void draw(DrawerPtr& d, float timestep);

	///	Called when a key down event is received.
	void keyDown(const std::wstring& action,
				 const std::wstring& keyPressed,
				 int keyModifier);

	///	Called when a mouse move event is received.
	void mouseMove(const TwoFloats& pos);

	///	Called to trigger the BarsState when we've reached a particular point in Music_PhaseDrift2.
	void reachedTime(const std::wstring& name, float time);

	///	Returns the name of this state.
	const std::wstring getName() const {return L"PhaseDriftState2";};
	///	Used to construct instances of this state.
	static GameStateCreator<PhaseDriftState2> creator;
  private:
	///	Helper method. Used to test whether a primitive matches one of the goal conditions.
	bool primitiveMatches(const std::wstring& image, const ThreeFloats& col);

	///	Struct representing one of the primitive shapes.
	struct Primitive
	{
		///	Which primitive it is.
		std::wstring image;
		///	The primitive's size (index to an inverted cos wave).
		float scale;
		///	The primitive's colour.
		ThreeFloats colour;
	};

	///	Drawing area size.
	TwoFloats drawSize;

	///	The current mouse position.
	TwoFloats mousePos;

	///	The current background colour.
	ThreeFloats backgroundColour;
	///	The desired background colour.
	ThreeFloats desiredColour;
	///	The background colours the player can move between.
	ThreeFloats potentialColours[8];
	///	Used to fade in from black.
	float fadeIn;

	///	The primitives.
	Primitive primitives[8];
	///	The available primitive images.
	std::vector<std::wstring> images;
	///	The current primitive image.
	std::wstring currentImage;
	///	Which primitive we're drawing first.
	int primitiveFirst;

	///	Used to determine how close the background colour is to one of the goal colours.
	float xCol;

	///	The index to the background music.
	int musicIndex;

	///	Used to flash the screen white when the player matches a goal condition.
	float whiteFlash;
	///	> 0 when we're ready to move to the next state.
	/*!
		0 = not ready for next state.
		1 = ready for <unnamed state>.
		2 = ready for BarsState.
	 */
	int nextStateReady;

	///	Our sounder.
	shared_ptr<PhaseDriftSounder> sounder;
};

#endif
