//	StoriesState.cpp - The 'Our Stories are Wrong' state.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "drawing/Drawer.h"
#include "StoriesState.h"
#include "MiscHelper.h"

#include <cmath>

using namespace std;

//------------------------------------------------------------------------------
GameStateCreator<StoriesState> StoriesState::creator;

//------------------------------------------------------------------------------
StoriesState::StoriesState(Application& app, SounderPtr& sound):
GameState(app, sound),
pointsIndex(0.0f),
radius(440.0f),
mousePos(1280.0f, 720.0f),
desiredPos(1280.0f, 720.0f),
justStarted(true)
{
	setMousePos(mousePos);
}

//------------------------------------------------------------------------------
StoriesState::~StoriesState()
{

}

//------------------------------------------------------------------------------
void StoriesState::update()
{
	int i;
	float angle;
	TwoFloats tempf;

	if(mousePos != desiredPos)
	{
		mousePos += (desiredPos - mousePos) * 0.05f;
		if(mousePos.getDistance(desiredPos) <= 1.0f)
			mousePos = desiredPos;
	}

	for(i=0;i<NumVectors;++i)
	{
		tempf.x = sinf(((float)i/(float)NumVectors) * 2.0f * NIALL_PI);
		tempf.y = cosf(((float)i/(float)NumVectors) * 2.0f * NIALL_PI);
		radius = 440.0f+(mousePos.y/(drawSize.y/360.0f));
		pointsPos[i] = centre + (tempf * radius);

		//Get non-rotated direction.
		tempf = (centre - pointsPos[i]).normal();

		//Rotate non-rotated direction according to the mouse x position.
		angle = (((mousePos.x/drawSize.x)*2.0f)-1.0f) * (0.5f * NIALL_PI); //0->90 degrees.
		pointsDir[i].x = (tempf.x * cosf(angle)) - (tempf.y * sinf(angle));
		pointsDir[i].y = (tempf.x * sinf(angle)) + (tempf.y * cosf(angle));
	}

	pointsIndex += 0.01f;
	if(pointsIndex > 1.0f)
		pointsIndex = 0.0f;
}

//------------------------------------------------------------------------------
void StoriesState::draw(DrawerPtr& d, float timestep)
{
	int i, j;
	float alpha;
	TwoFloats tempPos;
	const TwoFloats origin;
	const wstring tempstr(L"Point");
	const ThreeFloats pointCol(0.0f, 0.0f, 0.33f);
	ThreeFloats backColour(0.0f, 0.0f, 0.75f);

	if(justStarted)
	{
		TwoFloats tempf;

		centre = d->getDrawingAreaSize() * 0.5f;

		for(i=0;i<NumVectors;++i)
		{
			tempf.x = sinf(((float)i/(float)NumVectors) * 2.0f * NIALL_PI);
			tempf.y = cosf(((float)i/(float)NumVectors) * 2.0f * NIALL_PI);
			pointsPos[i] = centre + (tempf * (440.0f+(1280.0f/2.0f)));

			tempf = (centre - pointsPos[i]).normal();
			pointsDir[i].x = (tempf.x * cosf(0.5f * NIALL_PI)) - (tempf.y * sinf(0.5f * NIALL_PI));
			pointsDir[i].y = (tempf.x * sinf(0.5f * NIALL_PI)) + (tempf.y * cosf(0.5f * NIALL_PI));
		}

		justStarted = false;
	}

	if(drawSize == 0.0f)
		drawSize = d->getDrawingAreaSize();

	d->fillRect(origin, drawSize, backColour);

	for(i=0;i<NumVectors;++i)
	{
		tempPos = pointsPos[i] + ((pointsIndex * pointsDir[i]) * ((1.0f/(float)NumPoints) * radius));

		for(j=0;j<NumPoints;++j)
		{
			if(!j && (pointsIndex < 0.5f))
				alpha = pointsIndex;
			else if((j == (NumPoints-1)) && (pointsIndex > 0.5f))
				alpha = 0.5f-(pointsIndex-0.5f);
			else
				alpha = 0.5f;

			d->drawImage(tempstr, tempPos-(32.0f*0.5f), 0.5f, pointCol, alpha);

			tempPos += pointsDir[i] * ((1.0f/(float)NumPoints) * radius);
		}
	}
}

//------------------------------------------------------------------------------
void StoriesState::keyDown(const wstring& action,
						   const wstring& keyPressed,
						   int keyModifier)
{
	if(action == L"Escape")
		quit();
}

//------------------------------------------------------------------------------
void StoriesState::mouseMove(const TwoFloats& pos)
{
	/*int i;
	float angle;
	TwoFloats tempf;

	for(i=0;i<NumVectors;++i)
	{
		tempf.x = sinf(((float)i/(float)NumVectors) * 2.0f * NIALL_PI);
		tempf.y = cosf(((float)i/(float)NumVectors) * 2.0f * NIALL_PI);
		radius = 440.0f+(pos.y/(drawSize.y/360.0f));
		pointsPos[i] = centre + (tempf * radius);

		//Get non-rotated direction.
		tempf = (centre - pointsPos[i]).normal();

		//Rotate non-rotated direction according to the mouse x position.
		angle = (((pos.x/drawSize.x)*2.0f)-1.0f) * (0.5f * NIALL_PI); //0->90 degrees.
		pointsDir[i].x = (tempf.x * cosf(angle)) - (tempf.y * sinf(angle));
		pointsDir[i].y = (tempf.x * sinf(angle)) + (tempf.y * cosf(angle));
	}*/

	desiredPos = pos;
}
